@@ -29,17 +29,9 @@ fn run() -> proc_exit::ExitResult {
         }
     };
 
-    let colored = args.color.colored().or_else(git_stack::color::colored_env);
-    let mut colored_stdout = colored
-        .or_else(git_stack::color::colored_stdout)
-        .unwrap_or(true);
-    let mut colored_stderr = colored
-        .or_else(git_stack::color::colored_stderr)
-        .unwrap_or(true);
-    if (colored_stdout || colored_stderr) && !yansi::Paint::enable_windows_ascii() {
-        colored_stdout = false;
-        colored_stderr = false;
-    }
+    args.color.apply();
+    let colored_stdout = concolor_control::get(concolor_control::Stream::Stdout).ansi_color();
+    let colored_stderr = concolor_control::get(concolor_control::Stream::Stderr).ansi_color();
 
     git_stack::log::init_logging(args.verbose.clone(), colored_stderr);
 
