@@ -41,18 +41,20 @@ impl State {
         }
         let rebase = rebase;
 
-        let fixup = if args.fixup.is_some() || rebase {
-            repo_config.fixup()
-        } else {
-            // Assume the user is only wanting to show the tree and not modify it.
-            let no_op = git_stack::config::Fixup::Ignore;
-            if no_op != repo_config.fixup() {
-                log::trace!(
-                    "Ignoring `fixup={}` without an explicit `--rebase` or `--fixup`",
-                    repo_config.fixup()
-                );
+        let fixup = match (args.fixup, args.rebase) {
+            (Some(fixup), _) => fixup,
+            (_, true) => repo_config.auto_fixup(),
+            _ => {
+                // Assume the user is only wanting to show the tree and not modify it.
+                let no_op = git_stack::config::Fixup::Ignore;
+                if no_op != repo_config.auto_fixup() {
+                    log::trace!(
+                        "Ignoring `auto-fixup={}` without an explicit `--rebase`",
+                        repo_config.auto_fixup()
+                    );
+                }
+                no_op
             }
-            no_op
         };
         let push = args.push;
         let protected = git_stack::git::ProtectedBranches::new(
@@ -297,7 +299,7 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
     const STASH_STACK_NAME: &str = "git-stack";
     let mut success = true;
     let mut backed_up = false;
-    if state.rebase {
+    if state.rebase || state.fixup != git_stack::config::Fixup::Ignore {
         if stash_id.is_none() {
             stash_id = git_stack::git::stash_push(&mut state.repo, "branch-stash");
         }
@@ -327,7 +329,7 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
             .stacks
             .iter()
             .map(|stack| {
-                let script = plan_rebase(&state, stack).with_code(proc_exit::Code::FAILURE)?;
+                let script = plan_changes(&state, stack).with_code(proc_exit::Code::FAILURE)?;
                 if script.is_branch_deleted(&head_branch) {
                     head_branch = stack.onto.name.clone();
                 }
@@ -373,7 +375,7 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
     Ok(())
 }
 
-fn plan_rebase(state: &State, stack: &StackState) -> eyre::Result<git_stack::git::Script> {
+fn plan_changes(state: &State, stack: &StackState) -> eyre::Result<git_stack::git::Script> {
     let graphed_branches = stack.graphed_branches();
     let base_commit = state
         .repo
@@ -383,8 +385,10 @@ fn plan_rebase(state: &State, stack: &StackState) -> eyre::Result<git_stack::git
     let mut root = root.extend(&state.repo, git_stack::graph::Node::new(base_commit))?;
     git_stack::graph::protect_branches(&mut root, &state.repo, &state.protected_branches);
 
-    git_stack::graph::rebase_branches(&mut root, stack.onto.id);
-    git_stack::graph::drop_by_tree_id(&mut root);
+    if state.rebase {
+        git_stack::graph::rebase_branches(&mut root, stack.onto.id);
+        git_stack::graph::drop_by_tree_id(&mut root);
+    }
     git_stack::graph::fixup(&mut root, state.fixup);
 
     let script = git_stack::graph::to_script(&root);
@@ -428,8 +432,10 @@ fn show(state: &State, colored_stdout: bool) -> eyre::Result<()> {
 
             if state.dry_run {
                 // Show as-if we performed all mutations
-                git_stack::graph::rebase_branches(&mut root, stack.onto.id);
-                git_stack::graph::drop_by_tree_id(&mut root);
+                if state.rebase {
+                    git_stack::graph::rebase_branches(&mut root, stack.onto.id);
+                    git_stack::graph::drop_by_tree_id(&mut root);
+                }
                 git_stack::graph::fixup(&mut root, state.fixup);
             }
 
